@@ -11,6 +11,7 @@
 
 
 #include "dasher.h"
+#include "dasher_lock_dialogue.h"
 #ifdef WITH_MAEMO
 #include "dasher_maemo_helper.h"
 #endif
@@ -779,6 +780,8 @@ void dasher_main_setup_window_style(DasherMain *pSelf, bool bTopMost) {
   }
 
   gtk_window_set_keep_above(GTK_WINDOW(pPrivate->pMainWindow), pPrivate->bTopMost);
+  dasher_lock_dialogue_set_transient(GTK_WINDOW(pPrivate->pMainWindow));
+  
 
 #ifdef WITH_MAEMO
   gtk_window_set_accept_focus(GTK_WINDOW(pPrivate->pMainWindow), false);
@@ -1170,6 +1173,9 @@ extern "C" gboolean sidemenu_motion(GtkWidget *pWidget, GdkEventMotion *pEvent,
 extern "C" GdkFilterReturn keyboard_filter_cb(GdkXEvent *xevent, GdkEvent *event, gpointer data) {
   GtkDasherControl *pControl = (GtkDasherControl *)pDasherWidget;
 
+  if(!xevent)
+    return GDK_FILTER_CONTINUE;
+
   XEvent *xev = (XEvent *)xevent;
 
   if(xev->xany.type == KeyPress) {
@@ -1179,7 +1185,8 @@ extern "C" GdkFilterReturn keyboard_filter_cb(GdkXEvent *xevent, GdkEvent *event
     sKeyDetails.group = 0;
     sKeyDetails.level = 0;
 
-    gtk_dasher_control_external_key_down(pControl, gdk_keymap_lookup_key(0, &sKeyDetails));
+    if(pControl)
+      gtk_dasher_control_external_key_down(pControl, gdk_keymap_lookup_key(0, &sKeyDetails));
 
     return GDK_FILTER_REMOVE;
   }
@@ -1191,7 +1198,8 @@ extern "C" GdkFilterReturn keyboard_filter_cb(GdkXEvent *xevent, GdkEvent *event
     sKeyDetails.group = 0;
     sKeyDetails.level = 0;
 
-    gtk_dasher_control_external_key_up(pControl, gdk_keymap_lookup_key(0, &sKeyDetails));
+    if(pControl)
+      gtk_dasher_control_external_key_up(pControl, gdk_keymap_lookup_key(0, &sKeyDetails));
 
     return GDK_FILTER_REMOVE;
   }
