@@ -1,18 +1,18 @@
 #pragma once
 
-#include "../../DasherCore/DasherInterfaceBase.h"
+#include "../DasherCore/DasherInterfaceBase.h"
 
 class CSettingsStore;
 class CWinOptions;
 
-#include "WinOptions.h"
+#include "Common/WinOptions.h"
 
 class CDasherInterface:public Dasher::CDasherInterfaceBase {
 public:
   CDasherInterface(void);
   ~CDasherInterface(void);
 
-  CSettingsStore* CreateSettingsStore();
+  void CreateSettingsStore();
 
 // Functions below here are depreciated
   CWinOptions *GetSettingsStore() {
