@@ -12,6 +12,7 @@
 #include <Textstor.h>
 #include <Msctf.h>
 #include <Msaatext.h>
+#include <Htmlhelp.h>
 
 #include "DasherWindow.h"
 
@@ -57,13 +58,11 @@ CDasherWindow::CDasherWindow()
 	ATL::CWndClassInfo& wc = CDasherWindow::GetWndClassInfo();
 	wc.m_wc.hIcon = LoadIcon(WinHelper::hInstApp, (LPCTSTR) IDI_DASHER);
 	wc.m_wc.hCursor = LoadCursor(NULL, IDC_ARROW);
-  wc.m_wc.hbrBackground = (HBRUSH) (COLOR_ACTIVEBORDER + 1); // Must add one to the value we want for some unknown reason
-  wc.m_wc.lpszMenuName = (LPCTSTR) IDC_DASHER;
+    wc.m_wc.hbrBackground = (HBRUSH) (COLOR_ACTIVEBORDER + 1); // Must add one to the value we want for some unknown reason
+    wc.m_wc.lpszMenuName = (LPCTSTR) IDC_DASHER;
 //    wc.m_wc.lpszClassName = WndClassName;      // Not in a resource - does not require translation
 	wc.m_wc.hIconSm        = m_hIconSm;
 	
-  m_pKeyboardOutput = new CKeyboardOutput;
-
 #ifdef PJC_EXPERIMENTAL
 
 // Create an acc manager foobar
@@ -125,29 +124,15 @@ HWND CDasherWindow::Create()
   Tstring WindowTitle;
   WinLocalisation::GetResourceString(IDS_APP_TITLE, &WindowTitle);
 
-  // Create the Main window
-  //Tstring WndClassName = CreateMyClass();
-
-  // Create a CAppSettings
   m_pAppSettings = new CAppSettings(0, 0);
+  int iStyle(m_pAppSettings->GetLongParameter(APP_LP_STYLE));
 
   HWND hWnd;
-
-  if(m_pAppSettings->GetBoolParameter(APP_BP_KEYBOARD_MODE))
+  if((iStyle == 1) || (iStyle == 2))
     hWnd = CWindowImpl<CDasherWindow >::Create(NULL, NULL, WindowTitle.c_str(), WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN,  WS_EX_NOACTIVATE | WS_EX_APPWINDOW | WS_EX_TOPMOST);
   else
     hWnd = CWindowImpl<CDasherWindow >::Create(NULL, NULL, WindowTitle.c_str(), WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN);
 
-  // Splash screen (turned off for debugging when it gets in the way)
-  // It is deleted when Show() is called.
-  /*
-     D'oh I've moved things around. Now Splash being here is totally inappropriate;
-     Need to make it appear and disappear when training.
-     #ifndef _DEBUG
-     Splash = new CSplash(m_hwnd);
-     #endif
-   */
-
   // Create Widgets
   m_pDasher = new CDasher(hWnd);
 
@@ -155,7 +140,7 @@ HWND CDasherWindow::Create()
   m_pAppSettings->SetHwnd(hWnd);
   m_pAppSettings->SetDasher(m_pDasher);
 
-  m_pEdit = new CEdit();
+  m_pEdit = new CEdit(m_pAppSettings);
   m_pEdit->Create(hWnd, m_pAppSettings->GetBoolParameter(APP_BP_TIME_STAMP));
   m_pEdit->SetFont(m_pAppSettings->GetStringParameter(APP_SP_EDIT_FONT), m_pAppSettings->GetLongParameter(APP_LP_EDIT_FONT_SIZE));
 
@@ -163,7 +148,7 @@ HWND CDasherWindow::Create()
   g_hWnd = m_pEdit->GetHwnd();
 #endif
 
-  m_pToolbar = new CToolbar(hWnd, m_pDasher);
+  m_pToolbar = new CToolbar(hWnd, m_pAppSettings->GetBoolParameter(APP_BP_SHOW_TOOLBAR));
 
   // FIXME - the edit box really shouldn't need access to the interface, 
   // but at the moment it does, for training, blanking the display etc
@@ -173,10 +158,14 @@ HWND CDasherWindow::Create()
   // FIXME - we shouldn't need to know about these outside of CDasher
 
   m_pCanvas = m_pDasher->GetCanvas();
-  m_pSlidebar = m_pDasher->GetSlidebar();
+
+  m_pSlidebar = new CSlidebar(hWnd, m_pDasher);
+
 
   m_pSplitter = new CSplitter(this,100);
+  
   HWND hSplitter =  m_pSplitter->Create(hWnd);
+  
   if (!hSplitter)
 	  return 0;
 
@@ -202,6 +191,8 @@ HWND CDasherWindow::Create()
   m_pDasher->ConnectNode(-1, Dasher::CControlManager::CTL_USER+3, -2);
   m_pDasher->ConnectNode(Dasher::CControlManager::CTL_ROOT, Dasher::CControlManager::CTL_USER+3, -2);
 
+  m_pGameModeHelper = 0;
+
   return hWnd;
 }
 
@@ -216,22 +207,6 @@ CDasherWindow::~CDasherWindow() {
 }
 
 
-/////////////////////////////////////////////////////////////////////////////
-/*
-int CDasherWindow::MessageLoop() {
-  MSG msg;
-  while(GetMessage(&msg, NULL, 0, 0)) {
-    if(!TranslateAccelerator(msg.hwnd, hAccelTable, &msg)) {
-      TranslateMessage(&msg);
-      DispatchMessage(&msg);
-    }
-  }
-
-  return msg.wParam;
-}
-*/
-
-/////////////////////////////////////////////////////////////////////////////
 
 void CDasherWindow::Main()
 {
@@ -240,8 +215,6 @@ void CDasherWindow::Main()
 	Sleep(20); // limits framerate to 50fps
 }
 
-/////////////////////////////////////////////////////////////////////////////
-
 int CDasherWindow::MessageLoop() 
 {
 	MSG msg;
@@ -308,39 +281,24 @@ bool CDasherWindow::LoadWindowState() {
   return false;
 }
 
-
-
 void CDasherWindow::HandleParameterChange(int iParameter) {
   switch(iParameter) {
    case APP_BP_SHOW_TOOLBAR:
-    m_pToolbar->ShowToolbar(m_pAppSettings->GetBoolParameter(APP_BP_SHOW_TOOLBAR));
-    break;
+     m_pToolbar->ShowToolbar(m_pAppSettings->GetBoolParameter(APP_BP_SHOW_TOOLBAR));
+     break;
+   case APP_LP_STYLE:
+     Layout();
+     break;
    case APP_BP_TIME_STAMP:
-    m_pEdit->TimeStampNewFiles(m_pAppSettings->GetBoolParameter(APP_BP_TIME_STAMP));
-    break;
+     // TODO: reimplement
+     // m_pEdit->TimeStampNewFiles(m_pAppSettings->GetBoolParameter(APP_BP_TIME_STAMP));
+     break;
 	 case LP_MAX_BITRATE:
-	  m_pSlidebar->SetValue(m_pAppSettings->GetLongParameter(LP_MAX_BITRATE) / 100.0);
+    // TODO: reimplement
 	  break;
-   case BP_DASHER_PAUSED:
-    if(m_pAppSettings && m_pDasher && 
-       m_pAppSettings->GetBoolParameter(BP_DASHER_PAUSED) && 
-       m_pAppSettings->GetBoolParameter(APP_BP_COPY_ALL_ON_STOP))
-      if(m_pEdit)
-        m_pEdit->CopyAll();
-    break;
    case APP_SP_EDIT_FONT:
    case APP_LP_EDIT_FONT_SIZE:
-     m_pEdit->SetFont(m_pAppSettings->GetStringParameter(APP_SP_EDIT_FONT),m_pAppSettings->GetLongParameter(APP_LP_EDIT_FONT_SIZE));
-     break;
-   case APP_BP_KEYBOARD_MODE:
-     if(m_pAppSettings->GetBoolParameter(APP_BP_KEYBOARD_MODE)) {
-       SetWindowLong(GWL_EXSTYLE, GetWindowLong(GWL_EXSTYLE) | WS_EX_NOACTIVATE | WS_EX_APPWINDOW);
-       SetWindowPos(HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
-     }
-     else {
-       SetWindowLong(GWL_EXSTYLE, GetWindowLong(GWL_EXSTYLE) & !WS_EX_NOACTIVATE);
-       SetWindowPos(HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
-     }
+     m_pEdit->SetFont(m_pAppSettings->GetStringParameter(APP_SP_EDIT_FONT), m_pAppSettings->GetLongParameter(APP_LP_EDIT_FONT_SIZE));
      break;
   }
 }
@@ -444,118 +402,77 @@ LRESULT CDasherWindow::OnCommand(UINT message, WPARAM wParam, LPARAM lParam, BOO
 	// Parse the menu selections:
 	switch (wmId) 
 	{
-	  case ID_OPTIONS_ENTERTEXT:
-		  //GetWindowRect( &windowsize);
-		  //if(m_pEdit->GetTextEntry() == false) {
-			 // SetWindowPos(HWND_TOPMOST, windowsize.left, windowsize.top, (windowsize.right - windowsize.left), (windowsize.bottom - windowsize.top), NULL);
-			 // m_pEdit->TextEntry(true);
-		  //}
-		  //else 
-		  //{
-			 // SetWindowPos(HWND_NOTOPMOST, windowsize.left, windowsize.top, (windowsize.right - windowsize.left), (windowsize.bottom - windowsize.top), NULL);
-			 // m_pEdit->TextEntry(false);
-		  //}
-      m_pAppSettings->SetBoolParameter(APP_BP_KEYBOARD_MODE, !WinMenu.GetCheck(ID_OPTIONS_ENTERTEXT));
-		  return 0;
-	  case ID_OPTIONS_CONTROLMODE:
-		  m_pDasher->SetBoolParameter(BP_CONTROL_MODE, !WinMenu.GetCheck(ID_OPTIONS_CONTROLMODE));
-		  m_pDasher->RequestFullRedraw();
-		  return 0;
 	  case ID_OPTIONS_FONTSIZE_NORMAL:
 		  m_pDasher->SetLongParameter(LP_DASHER_FONTSIZE, Dasher::Opts::FontSize(1));
-		  m_pDasher->RequestFullRedraw();
 		  break;
 	  case ID_OPTIONS_FONTSIZE_LARGE:
 		  m_pDasher->SetLongParameter(LP_DASHER_FONTSIZE, Dasher::Opts::FontSize(2));
-		  m_pDasher->RequestFullRedraw();
 		  break;
 	  case ID_OPTIONS_FONTSIZE_VERYLARGE:
 		  m_pDasher->SetLongParameter(LP_DASHER_FONTSIZE, Dasher::Opts::FontSize(4));
-		  m_pDasher->RequestFullRedraw();
 		  break;
 	  case ID_OPTIONS_EDITFONT:{
 		  CHOOSEFONT Data;
 		  LOGFONT lf;
-		  HFONT Font = (HFONT) GetStockObject(DEFAULT_GUI_FONT);
-		  GetObject(Font, sizeof(LOGFONT), &lf);
+      HFONT Font = (HFONT) GetStockObject(DEFAULT_GUI_FONT);
+      GetObject(Font, sizeof(LOGFONT), &lf);
+      Tstring tstrFaceName;
+      WinUTF8::UTF8string_to_wstring(m_pAppSettings->GetStringParameter(APP_SP_EDIT_FONT), tstrFaceName);
+      _tcscpy(lf.lfFaceName, tstrFaceName.c_str());
+      lf.lfHeight = m_pAppSettings->GetLongParameter(APP_LP_EDIT_FONT_SIZE);
 		  Data.Flags = CF_INITTOLOGFONTSTRUCT | CF_SCREENFONTS;
 		  Data.lStructSize = sizeof(CHOOSEFONT);
 		  Data.hwndOwner = m_hWnd;
 		  Data.lpLogFont = &lf;
-		  ChooseFont(&Data);
-		  string FontName;
-		  WinUTF8::wstring_to_UTF8string(lf.lfFaceName, FontName);
-		  m_pAppSettings->SetStringParameter(APP_SP_EDIT_FONT, FontName);
-      m_pAppSettings->SetLongParameter(APP_LP_EDIT_FONT_SIZE, lf.lfHeight);
-							   }
-							   break;
+      if(ChooseFont(&Data)) {
+		    string FontName;
+		    WinUTF8::wstring_to_UTF8string(lf.lfFaceName, FontName);
+		    m_pAppSettings->SetStringParameter(APP_SP_EDIT_FONT, FontName);
+        m_pAppSettings->SetLongParameter(APP_LP_EDIT_FONT_SIZE, lf.lfHeight);
+      }
+	    }
+			break;
 	  case ID_OPTIONS_DASHERFONT:
 		  {
 		  CHOOSEFONT Data;
 		  LOGFONT lf;
 		  HFONT Font = (HFONT) GetStockObject(DEFAULT_GUI_FONT);
 		  GetObject(Font, sizeof(LOGFONT), &lf);
+      Tstring tstrFaceName;
+      WinUTF8::UTF8string_to_wstring(m_pAppSettings->GetStringParameter(SP_DASHER_FONT), tstrFaceName);
+      _tcscpy(lf.lfFaceName, tstrFaceName.c_str());
 		  Data.Flags = CF_INITTOLOGFONTSTRUCT | CF_SCREENFONTS;
 		  Data.lStructSize = sizeof(CHOOSEFONT);
 		  Data.hwndOwner = m_hWnd;
 		  Data.lpLogFont = &lf;
-		  ChooseFont(&Data);
-		  string FontName;
-		  WinUTF8::wstring_to_UTF8string(lf.lfFaceName, FontName);
-		  m_pAppSettings->SetStringParameter(SP_DASHER_FONT, FontName);
+      if(ChooseFont(&Data)) {
+		    string FontName;
+ 		    WinUTF8::wstring_to_UTF8string(lf.lfFaceName, FontName);
+  		  m_pAppSettings->SetStringParameter(SP_DASHER_FONT, FontName);
+      }
 		 }
 		 return 0;
 	  case ID_OPTIONS_RESETFONT:
 		  m_pAppSettings->ResetParamater(SP_DASHER_FONT);
 		  m_pAppSettings->ResetParamater(APP_SP_EDIT_FONT);
 		  return 0;
-	  case IDM_ABOUT:
-		  {
+	  case IDM_ABOUT: {
 			  CAboutBox Aboutbox;
 			  Aboutbox.DoModal(m_hWnd);
 			  return 0;
 		  }
-	  case ID_OPTIONS_EDITKEYS:
-		  {
-			  CKeyBox KeyBox(m_hWnd, m_pDasher);
-			  return 0;
-		  }
-	  case ID_OPTIONS_ALPHABET:
-		  {
-			  CAlphabetBox AlphabetBox(m_hWnd, m_pDasher);
-			  return 0;
-		  }
-	  case ID_OPTIONS_COLOURS:
-		  {
-			  CColourBox ColourBox(m_hWnd, m_pDasher);
-		  return 0;
-		  }
-	  case ID_OPTIONS_PREFS:
-		  {
-		  CPrefs Prefs(m_hWnd, m_pDasher, m_pAppSettings);
-		  return 0;
+	  case ID_OPTIONS_PREFS: {
+		    CPrefs Prefs(m_hWnd, m_pDasher, m_pAppSettings);
+		    return 0;
 		  }
 	  case ID_HELP_CONTENTS:
-		  WinHelp(TEXT("dasher.hlp"), HELP_FINDER, 0);
+      HtmlHelp(m_hWnd, L"Dasher.chm", HH_DISPLAY_INDEX, NULL);
 		  return 0;
+    case ID_HELP_DASHERTUTORIAL:
+      m_pGameModeHelper = new CGameModeHelper(m_pDasher);
+      return 0;
 	  case IDM_EXIT:
-		  //SendMessage(m_hWnd, WM_CLOSE, 0, 0);
-		  if(m_pEdit->QueryClose())
-        DestroyWindow();
-		  return 0;
-	
-		  // FIXME - These options shouldn't pass through the interface
-	  case ID_TB_SHOW:
-		  //     m_pDasher->SetBoolParameter(BP_SHOW_TOOLBAR, !WinMenu.GetCheck(ID_TB_SHOW));
-		  //    m_pToolbar->ShowToolbar(m_pDasher->GetBoolParameter(BP_SHOW_TOOLBAR));
-		  return 0;
-	  case ID_TB_TEXT:
-		  //    m_pDasher->SetBoolParameter(BP_SHOW_TOOLBAR_TEXT, !WinMenu.GetCheck(ID_TB_TEXT));
-		  //    m_pToolbar->ShowToolbar(m_pDasher->GetBoolParameter(BP_SHOW_TOOLBAR));
-		  return 0;
-	  case ID_TB_LARGE:
-		  //    m_pDasher->SetBoolParameter(BP_SHOW_LARGE_ICONS, !WinMenu.GetCheck(ID_TB_LARGE));
-		  //    m_pToolbar->ShowToolbar(m_pDasher->GetBoolParameter(BP_SHOW_TOOLBAR));
+		  DestroyWindow();
 		  return 0;
 	  case ID_EDIT_SELECTALL:
 		  if(m_pEdit)
@@ -600,69 +517,15 @@ LRESULT CDasherWindow::OnCommand(UINT message, WPARAM wParam, LPARAM lParam, BOO
 		  if(m_pEdit)
 			  m_pEdit->SaveAs();
 		  return 0;
+    case ID_FILE_APPEND:
+      if(m_pEdit)
+        m_pEdit->SaveAs();
+      return 0;
 	  case ID_IMPORT_TRAINFILE:
-		  m_pDasher->TrainFile(m_pEdit->Import());
+      // TODO: Fix dummy arguments
+		  m_pDasher->TrainFileWrapper(m_pEdit->Import(),0,0);
 		  return 0;
-	
-		  // FIXME - This options shouldn't passs through the interface
-
-	  case ID_FIX_SPLITTER:
-		  //     m_pDasher->SetBoolParameter(BP_FIX_LAYOUT, !WinMenu.GetCheck(ID_FIX_SPLITTER));
-		  return 0;
-	  case ID_SHOW_SLIDE:
-		  m_pDasher->SetBoolParameter(BP_SHOW_SLIDER, !WinMenu.GetCheck(ID_SHOW_SLIDE));
-		  return 0;
-	
-		  // FIXME - These options shouldn't pass through the interface
-	  case ID_TIMESTAMP:
-		  //  m_pDasher->SetBoolParameter(BP_TIME_STAMP, !WinMenu.GetCheck(ID_TIMESTAMP));
-		  return 0;
-	  case ID_COPY_ALL_ON_STOP:
-		  //  m_pDasher->SetBoolParameter(BP_COPY_ALL_ON_STOP, !WinMenu.GetCheck(ID_COPY_ALL_ON_STOP));
-		  return 0;
-	
-	  case ID_ORIENT_ALPHABET:
-	  case ID_ORIENT_LR:
-	  case ID_ORIENT_RL:
-	  case ID_ORIENT_TB:
-	  case ID_ORIENT_BT:
-		  if(wmId==ID_ORIENT_ALPHABET) {
-			  m_pDasher->SetLongParameter(LP_ORIENTATION, Opts::Alphabet);
-		  } else if(wmId==ID_ORIENT_LR) {
-			  m_pDasher->SetLongParameter(LP_ORIENTATION, Opts::LeftToRight);
-		  } else if(wmId==ID_ORIENT_RL) {
-			  m_pDasher->SetLongParameter(LP_ORIENTATION, Opts::RightToLeft);
-		  } else if(wmId==ID_ORIENT_TB) {
-			  m_pDasher->SetLongParameter(LP_ORIENTATION, Opts::TopToBottom);
-		  } else if(wmId==ID_ORIENT_BT) {
-			  m_pDasher->SetLongParameter(LP_ORIENTATION, Opts::BottomToTop);
-		  } else {  // If not any of these, this is the default setting
-			  m_pDasher->SetLongParameter(LP_ORIENTATION, Opts::LeftToRight);
-		  }
-		  return 0;
-	
-		  // FIXME - These options shouldn't pass through the interface
-		  // !!! Need to make this work. Probably get interface to tell edit box how to save and screen how to display
-		  // case ID_SAVE_AS_USER_CODEPAGE:
-		  // case ID_SAVE_AS_ALPHABET_CODEPAGE:
-		  // case ID_SAVE_AS_UTF8:
-		  // case ID_SAVE_AS_UTF16_LITTLE:
-		  // case ID_SAVE_AS_UTF16_BIG:
-		  //   if(wmId==ID_SAVE_AS_USER_CODEPAGE) {
-		  ////     m_pDasher->SetLongParameter(LP_FILE_ENCODING, Opts::UserDefault);
-		  //   } else if(wmId==ID_SAVE_AS_ALPHABET_CODEPAGE) {
-		  //  //   m_pDasher->SetLongParameter(LP_FILE_ENCODING, Opts::AlphabetDefault);
-		  //   } else if(wmId==ID_SAVE_AS_UTF8) {
-		  //     m_pDasher->SetLongParameter(LP_FILE_ENCODING, Opts::UTF8);
-		  //   } else if(wmId==ID_SAVE_AS_UTF16_LITTLE) {
-		  //     m_pDasher->SetLongParameter(LP_FILE_ENCODING, Opts::UTF16LE);
-		  //   } else if(wmId==ID_SAVE_AS_UTF16_BIG) {
-		  //     m_pDasher->SetLongParameter(LP_FILE_ENCODING, Opts::UTF16BE);
-		  //   } else {// If not any of these, this is the default setting
-		  //     m_pDasher->SetLongParameter(LP_FILE_ENCODING, Opts::UserDefault);
-		  //   }
-		  //   break;
-	  default:
+    default:
 		  return DefWindowProc(message, wParam, lParam);
 	}
 
@@ -671,96 +534,62 @@ LRESULT CDasherWindow::OnCommand(UINT message, WPARAM wParam, LPARAM lParam, BOO
 	return 0;
 }
 
-LRESULT CDasherWindow::OnTimer(UINT message, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
-  bHandled = TRUE;
+LRESULT CDasherWindow::OnDasherEvent(UINT message, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
+	// Apparently putting the typecast directly in the switch doesn't work
+  CEvent *pEvent( (CEvent *)lParam );
 
-  HWND testwindow;
-  testwindow = GetForegroundWindow();
-  
-  if(testwindow != m_hWnd) {
-    if(!m_bHaveTarget || (testwindow != m_hTargetWindow)) {
-      m_hTargetWindow = testwindow;
-      AttachThreadInput(GetCurrentThreadId(),GetWindowThreadProcessId(m_hTargetWindow,NULL),TRUE);
-      m_bHaveTarget = true;
-    }
+  switch(pEvent->m_iEventType) {
+    case EV_PARAM_NOTIFY:
+      HandleParameterChange(((CParameterNotificationEvent *)pEvent)->m_iParameter);
+      break;
+    case EV_CONTROL:
+      HandleControlEvent(((CControlEvent *)pEvent)->m_iID);
+      break;
+    case EV_EDIT: 
+      if(m_pGameModeHelper) {
+        Dasher::CEditEvent * pEvt(static_cast< Dasher::CEditEvent * >(pEvent));
+
+        switch (pEvt->m_iEditType) {
+          case 1:
+            m_pGameModeHelper->Output(pEvt->m_sText);
+            break;
+          case 2:
+            m_pGameModeHelper->Delete(pEvt->m_sText.size());
+            break;
+        }
+      }
+      break;
+
+
+    default:
+      break;
   }
   
-  return 0;
-}
-
-LRESULT CDasherWindow::OnShowWindow(UINT message, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
-  bHandled = TRUE;
+  if(m_pEdit) 
+     m_pEdit->HandleEvent(pEvent);
   
- /* if(m_pAppSettings->GetBoolParameter(APP_BP_KEYBOARD_MODE)) {
-    SetWindowLong(GWL_EXSTYLE, GetWindowLong(GWL_EXSTYLE) | WS_EX_NOACTIVATE | WS_EX_APPWINDOW );
-    SetWindowPos(HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
-  }*/
-
-  return 0;
-}
-
-LRESULT CDasherWindow::OnDasherEvent(UINT message, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
-{
-	bHandled = TRUE;
-
-	// We can't switch on a dynamically allocated value...
-    CEvent *pEvent( (CEvent *)lParam );
-
-    switch(pEvent->m_iEventType) {
-      case EV_PARAM_NOTIFY:
-        HandleParameterChange(((CParameterNotificationEvent *)pEvent)->m_iParameter);
-        break;
-      case EV_EDIT:
-        if(m_pEdit)
-          m_pEdit->HandleEvent(pEvent);
-        if(m_pAppSettings->GetBoolParameter(APP_BP_KEYBOARD_MODE) && (GetForegroundWindow() != m_hWnd))
-          m_pKeyboardOutput->HandleEvent(pEvent);
-        break;
-      case EV_EDIT_CONTEXT:
-        if(m_pEdit && m_pDasher)
-          m_pDasher->SetContext(m_pEdit->GetContext(((CEditContextEvent*)pEvent)->m_iMaxLength));
-        break;
-      case EV_START:
-        break;
-      case EV_STOP:
-        if(m_pAppSettings->GetBoolParameter(APP_BP_SPEECH_MODE) && m_pEdit)
-          m_pEdit->speak(2);
-        break;
-      case EV_CONTROL:
-        HandleControlEvent(((CControlEvent *)pEvent)->m_iID);
-        break;
-      default:
-        break;
-    }
 	return 0;
 }
 
-LRESULT CDasherWindow::OnDasherFocus(UINT message, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
-{
-	bHandled = TRUE;
+LRESULT CDasherWindow::OnDasherFocus(UINT message, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
 	::SetFocus(m_pEdit->GetHwnd());
-    HWND *pHwnd((HWND *)lParam);
-    m_pEdit->SetKeyboardTarget(*pHwnd);
+
+  // TODO: Is this obsolete?
+  HWND *pHwnd((HWND *)lParam);
+  m_pEdit->SetKeyboardTarget(*pHwnd);
 	return 0;
 }
 
-
-
-LRESULT CDasherWindow::OnDestroy(UINT message, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
-{
-	bHandled = TRUE;
-	OutputDebugString(TEXT("DasherWindow WM_DESTROY\n"));
-  //if(m_pEdit)
-	 // m_pEdit->New();
+LRESULT CDasherWindow::OnDestroy(UINT message, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
 	if(m_pEdit != 0) {
 		m_pEdit->write_to_file();
 	}
-	PostQuitMessage(0);
+
+  PostQuitMessage(0);
 	return 0;
 }
 
-LRESULT CDasherWindow::OnGetMinMaxInfo(UINT message, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
-{
+LRESULT CDasherWindow::OnGetMinMaxInfo(UINT message, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
 	// not yet created
 	if (m_pToolbar == 0 || m_pSplitter == 0 || m_pSlidebar == 0)
 		return 0;
@@ -770,100 +599,92 @@ LRESULT CDasherWindow::OnGetMinMaxInfo(UINT message, WPARAM wParam, LPARAM lPara
     lppt = (LPPOINT) lParam;    // lParam points to array of POINTs
     lppt[3].x = 100;            // Set minimum width (arbitrary)
     // Set minimum height:
-    lppt[3].y = m_pToolbar->Resize() + m_pSplitter->GetPos()
-      + m_pSplitter->GetHeight() + m_pSlidebar->GetHeight() + GetSystemMetrics(SM_CYEDGE) * 10;
+    if(m_pAppSettings->GetBoolParameter(APP_BP_SHOW_TOOLBAR))
+      lppt[3].y = m_pToolbar->GetHeight() + m_pSplitter->GetPos()
+        + m_pSplitter->GetHeight() + m_pSlidebar->GetHeight() + GetSystemMetrics(SM_CYEDGE) * 10;
+    else
+      lppt[3].y = m_pSplitter->GetPos()
+        + m_pSplitter->GetHeight() + m_pSlidebar->GetHeight() + GetSystemMetrics(SM_CYEDGE) * 10;
 
 	return 0;
 }
 
-LRESULT CDasherWindow::OnInitMenuPopup(UINT message, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
-{
-	bHandled = TRUE; 
+LRESULT CDasherWindow::OnInitMenuPopup(UINT message, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
 	PopulateSettings();
 	WinMenu.SortOut((HMENU) wParam);
 	return 0;
 }
 
-LRESULT CDasherWindow::OnClose(UINT message, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
-{
-	bHandled = TRUE;
-  if(m_pEdit->QueryClose()) {
-	  SaveWindowState();
-    DestroyWindow();
-  }
-  return 0;
+LRESULT CDasherWindow::OnClose(UINT message, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
+  // TODO: Prompt for confirmation here
+	SaveWindowState();
+  DestroyWindow();
+	return 0;
 }
 
-LRESULT CDasherWindow::OnSize(UINT message, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
-{
-	bHandled = TRUE;
+LRESULT CDasherWindow::OnSize(UINT message, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
 	if(wParam == SIZE_MINIMIZED)
-      return 0;
-//    m_pDasher->SetLongParameter(LP_SCREEN_WIDTH, LOWORD(lParam));
- //   m_pDasher->SetLongParameter(LP_SCREEN_HEIGHT, HIWORD(lParam));
-    Layout();
+    return 0;
+    
+  m_pToolbar->Resize();
+  m_pSlidebar->Resize();
+
+  Layout();
 
 	return 0;
 }
 
 
-LRESULT CDasherWindow::OnSetFocus(UINT message, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
-{
-	bHandled = TRUE; 
-  if(m_pCanvas)
-    ::SetFocus(m_pCanvas->getwindow());
+LRESULT CDasherWindow::OnSetFocus(UINT message, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
+	::SetFocus(m_pCanvas->getwindow());
  	return 0;
 }
 
-LRESULT CDasherWindow::OnDrawItem(UINT message, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
-{
-	bHandled = TRUE; 
-	if(((LPDRAWITEMSTRUCT)lParam)->hwndItem == m_pSlidebar->getwindow()) 
-	{
-      m_pSlidebar->Redraw((LPDRAWITEMSTRUCT)lParam);
-    }
-    else
-      return DefWindowProc( message, wParam, lParam); 
- 
-	return 0;
-}
-
-LRESULT CDasherWindow::OnOther(UINT message, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
-{
+LRESULT CDasherWindow::OnOther(UINT message, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
 	if (message == WM_DASHER_EVENT)
 		return OnDasherEvent( message, wParam, lParam, bHandled);
 	else if (message == WM_DASHER_FOCUS)
 		return OnDasherFocus(message, wParam, lParam, bHandled);
-	
+  else if (message == DASHER_SHOW_PREFS) {
+    CPrefs Prefs(m_hWnd, m_pDasher, m_pAppSettings);
+  }
 	return 0;
-
 }
 
-//LRESULT CDasherWindow::WndProc(HWND Window, UINT message, WPARAM wParam, LPARAM lParam) {
-  //RECT windowsize;
-//  switch (message) {
-  //  case MY_LAYOUT:
-  //  Layout();
- //   break;
+void CDasherWindow::Layout() {
+  int iStyle(m_pAppSettings->GetLongParameter(APP_LP_STYLE));
+  
+  // Set up the window properties
 
+  if((iStyle == 1) || (iStyle == 2)) {
+    SetWindowLong(GWL_EXSTYLE, GetWindowLong(GWL_EXSTYLE) | WS_EX_NOACTIVATE | WS_EX_APPWINDOW);
+    SetWindowPos(HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
+  }
+  else {
+    SetWindowLong(GWL_EXSTYLE, GetWindowLong(GWL_EXSTYLE) & !WS_EX_NOACTIVATE);
+    SetWindowPos(HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
+  }
+
+  // Now do the actual layout
+
+  bool bHorizontal(iStyle == 1); 
+  bool bShowEdit(iStyle != 2);
 
-void CDasherWindow::Layout() 
-{
   RECT ClientRect;
   GetClientRect( &ClientRect);
   const int Width = ClientRect.right;
   const int Height = ClientRect.bottom;
 
   int ToolbarHeight;
-  if(m_pToolbar != 0)
-    ToolbarHeight = m_pToolbar->Resize();
+  if((m_pToolbar != 0) && m_pAppSettings->GetBoolParameter(APP_BP_SHOW_TOOLBAR))
+    ToolbarHeight = m_pToolbar->GetHeight() + 2;
   else
     ToolbarHeight = 0;
   int CurY = ToolbarHeight;
 
   int SlidebarHeight;
   if(m_pSlidebar != 0)
-    SlidebarHeight = m_pSlidebar->Resize(Width, Height);
+    SlidebarHeight = m_pSlidebar->GetHeight();
   else
     SlidebarHeight = 0;
   int MaxCanvas = Height - SlidebarHeight;
@@ -875,14 +696,33 @@ void CDasherWindow::Layout()
     SplitterPos = min(SplitterPos, MaxCanvas - 3 * SplitterHeight);
     m_pSplitter->Move(SplitterPos, Width);
 
-    if(m_pEdit)
-      m_pEdit->Move(0, CurY, Width, SplitterPos - CurY);
+    if(bHorizontal) {
+      if(m_pCanvas)
+        m_pCanvas->Move(0, CurY, Width / 2, MaxCanvas - CurY);
 
-    CurY = SplitterPos + SplitterHeight;
-    int CanvasHeight = Height - CurY - SlidebarHeight - GetSystemMetrics(SM_CYEDGE);
+      if(m_pEdit)
+        m_pEdit->Move(Width / 2, CurY, Width / 2, MaxCanvas - CurY);
+    }
+    else {
+      if(bShowEdit) {
+        m_pEdit->ShowWindow(SW_SHOW);
+        m_pSplitter->ShowWindow(SW_SHOW);
+
+        if(m_pEdit)
+          m_pEdit->Move(0, CurY, Width, SplitterPos - CurY);
+
+        CurY = SplitterPos + SplitterHeight;
+      }
+      else {
+        m_pEdit->ShowWindow(SW_HIDE);
+        m_pSplitter->ShowWindow(SW_HIDE);
+      }
+
+      int CanvasHeight = Height - CurY - SlidebarHeight - GetSystemMetrics(SM_CYEDGE);
  
-    if(m_pCanvas)
-      m_pCanvas->Move(0, CurY, Width, CanvasHeight);
+      if(m_pCanvas)
+        m_pCanvas->Move(0, CurY, Width, CanvasHeight);
+     }
   }
 }
 
@@ -891,19 +731,9 @@ void CDasherWindow::PopulateSettings() {
     WinMenu.SetStatus(ID_OPTIONS_FONTSIZE_NORMAL, false, m_pDasher->GetLongParameter(LP_DASHER_FONTSIZE)==1);
     WinMenu.SetStatus(ID_OPTIONS_FONTSIZE_LARGE, false, m_pDasher->GetLongParameter(LP_DASHER_FONTSIZE)==2);
     WinMenu.SetStatus(ID_OPTIONS_FONTSIZE_VERYLARGE, false, m_pDasher->GetLongParameter(LP_DASHER_FONTSIZE)==4);
-
-    WinMenu.SetStatus(ID_OPTIONS_ENTERTEXT, false, m_pAppSettings->GetBoolParameter(APP_BP_KEYBOARD_MODE));
-    WinMenu.SetStatus(ID_OPTIONS_CONTROLMODE, false, m_pDasher->GetBoolParameter(BP_CONTROL_MODE));
-
-    WinMenu.SetStatus(ID_ORIENT_ALPHABET, false, m_pDasher->GetLongParameter(LP_ORIENTATION)==Opts::Alphabet);
-    WinMenu.SetStatus(ID_ORIENT_LR,       false, m_pDasher->GetLongParameter(LP_ORIENTATION)==Opts::LeftToRight);
-    WinMenu.SetStatus(ID_ORIENT_RL,       false, m_pDasher->GetLongParameter(LP_ORIENTATION)==Opts::RightToLeft);
-    WinMenu.SetStatus(ID_ORIENT_TB,       false, m_pDasher->GetLongParameter(LP_ORIENTATION)==Opts::TopToBottom);
-    WinMenu.SetStatus(ID_ORIENT_BT,       false, m_pDasher->GetLongParameter(LP_ORIENTATION)==Opts::BottomToTop);
   }
 }
 
-
 #ifdef PJC_EXPERIMENTAL
 
 VOID CALLBACK WinEventProc(HWINEVENTHOOK hWinEventHook, DWORD event, HWND hwnd, LONG idObject, LONG idChild, DWORD dwEventThread, DWORD dwmsEventTime) {
